import time
import click

from zec2 import get_aws_objects, get_instance_by_number, \
                 run_action, list_instances


@click.group()
@click.pass_context
@click.option('-p', '--aws_profile', help='AWS API profile')
def cli(context, aws_profile):
    ec2, vpcs = get_aws_objects(aws_profile)
    ec2.connection()
    context.obj = {
        'aws_profile': aws_profile,
        'objects': {
            'ec2': ec2,
            'vpcs': vpcs
        }
    }


@cli.command()
@click.pass_context
@click.option('-f/-no-f', default=False, help='Run ls forever')
def ls(context, f):
    if f:
        while True:
            click.clear()
            list_instances(**context.obj.get('objects'))
            time.sleep(2)
    else:
        list_instances(**context.obj.get('objects'))


@cli.command()
@click.pass_context
@click.argument('number')
@click.option('-u', '--user', help='SSH user')
@click.option('-i', '--key_path', help='AWS Key Pair')
def ssh(context, number, user, key_path):
    instance = get_instance_by_number(number, **context.obj.get('objects'))
    click.echo(instance.ssh_command(user, key_path))


@cli.command()
@click.pass_context
@click.argument('number')
def start(context, number):
    run_action(number, 'start', **context.obj.get('objects'))


@cli.command()
@click.pass_context
@click.argument('number')
def stop(context, number):
    run_action(number, 'stop', **context.obj.get('objects'))


@cli.command()
@click.pass_context
@click.argument('number')
def restart(context, number):
    run_action(number, 'restart', **context.obj.get('objects'))


@cli.command()
@click.pass_context
@click.argument('number')
def terminate(context, number):
    run_action(number, 'terminate', **context.obj.get('objects'))



import boto3
from click import UsageError
from botocore import exceptions


class AwsClient(object):

    conn = None

    def __init__(self):
        self._aws_profile = 'default'

    def aws_profile(self, _aws_profile):
        self._aws_profile = _aws_profile

    def connection(self):
        try:
            s = boto3.Session(profile_name=self._aws_profile)
            return s.resource('ec2')
        except exceptions.ProfileNotFound as e:
            raise UsageError(str(e))
        except exceptions.ConfigParseError as e:
            raise UsageError(str(e))
        except exceptions.ConfigNotFound as e:
            raise UsageError(str(e))
        except exceptions.UnknownCredentialError as e:
            raise UsageError(str(e))
        except exceptions.NoRegionError as e:
            raise UsageError(str(e))

    @property
    def client(self):
        if self.conn is None:
            self.conn = self.connection()
        return self.conn


class AwsEc2(AwsClient):

    def list(self, _filter=None):

        if _filter is None:
            _filter = []

        try:
            instances = self.client.instances.filter(Filters=[
                {
                    'Name': name,
                    'Values': value if type(value) == list else [value]
                }
                for name, value in _filter
            ])

            def by_name(instance):
                for tag in instance.tags or []:
                    if tag['Key'] == 'Name':
                        return tag['Value']
                else:
                    return ''

            instances = sorted(instances, key=by_name)
        except exceptions.ClientError as e:
            raise UsageError(str(e))

        return [AwsEc2Instance(instance) for instance in instances]


class AwsVpc(AwsClient):

    def list(self):
        return self.client.vpcs.all()


class AwsEc2Instance(object):

    states = {
        'running': 'green',
        'stopping': 'yellow',
        'stopped': 'red',
        'pending': 'yellow',
        'shutting-down': 'red'
    }

    def __init__(self, instance):
        self._instance = instance

    def name(self):
        for tag in self._instance.tags or []:
            if tag['Key'] == 'Name':
                return tag['Value']
        else:
            return ''

    def state(self):
        state = self._instance.state['Name']
        state_fg = self.states.get(state, 'red')
        return state, state_fg

    def zone(self):
        return str(self._instance.placement['AvailabilityZone'])

    def public_ip_address(self):
        if self._instance.public_ip_address:
            return self._instance.public_ip_address
        else:
            return ''

    def ssh_ip_address(self):
        if self._instance.public_ip_address:
            return self._instance.public_ip_address
        else:
            return self._instance.private_ip_address

    @property
    def aws(self):
        return self._instance

    def ssh_command(self, user, key_path):
        if not user:
            user = 'ec2-user'
        if not key_path:
            key_path = '~/.ssh/%s.pem' % self._instance.key_name
        params = (key_path, user, self.ssh_ip_address())
        return 'ssh -i %s %s@%s' % params

    def stop(self):
        self._instance.stop()

    def start(self):
        self._instance.start()

    def restart(self):
        self._instance.reboot()

    def terminate(self):
        self._instance.terminate()

    def disable_api_termination(self):
        self._instance.modify_attribute(DisableApiTermination={'Value': False})



from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='zec2',
    version="0.3.1",

    description='Easily ssh to your AWS EC2 instances',
    long_description=readme(),
    long_description_content_type='text/markdown',

    url='https://github.com/arrrlo/zec2',
    licence='MIT',

    author='Ivan Arar',
    author_email='ivan.arar@gmail.com',

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='amazon, aws, ec2, ssh, instance',

    packages=['zec2'],
    install_requires=[
        'click>=7.0',
        'six>=1.14.0',
        'boto3~=1.7.69',
        'terminaltables~=3.1.0'
    ],

    entry_points={
        'console_scripts': [
            'zec2=zec2.cli:cli'
        ],
    },

    project_urls={
        'Source': 'https://github.com/arrrlo/zec2',
    },
)



import click
from click import UsageError
from botocore import exceptions
from terminaltables import AsciiTable

from zec2.aws_ec2 import AwsEc2, AwsVpc


def list_instances(vpcs, ec2):
    i = 0
    tables = []
    for vpc in vpcs.list():
        instances = ec2.list(_filter=[('vpc-id', vpc.id)])
        table_data, i = instance_table_maker(instances, i=i)
        table = AsciiTable(table_data)
        tables.append((vpc.id, table.table))

    for vpc, table in tables:
        click.echo()
        click.secho(vpc, fg='yellow')
        click.echo(table)

    click.echo()


def instance_table_maker(instances, table_data=None, i=None):
    if type(instances) is not list:
        instances = [instances]

    if table_data is None:
        table_data = [
            ['', 'Name', 'Instance ID', 'Private IP', 'Public IP',
             'State', 'Key Pair', 'Type', 'Launch time', 'Zone'],
        ]

    if i is None:
        i = 0

    for instance in instances:
        row = list()

        row.append(str(i + 1))
        row.append(click.style(instance.name(), fg='yellow'))
        row.append(instance.aws.id)
        row.append(instance.aws.private_ip_address)
        row.append(instance.public_ip_address())
        state, state_fg = instance.state()
        row.append(click.style(state, fg=state_fg))
        row.append(instance.aws.key_name)
        row.append(instance.aws.instance_type)
        row.append(instance.aws.launch_time)
        row.append(instance.zone())

        table_data.append(row)

        i += 1

    return table_data, i


def get_aws_objects(aws_profile):
    ec2 = AwsEc2()
    vpcs = AwsVpc()

    if aws_profile:
        ec2.aws_profile(aws_profile)
        vpcs.aws_profile(aws_profile)

    return ec2, vpcs


def get_instance_by_number(number, ec2, vpcs):
    instances = list()
    for vpc in vpcs.list():
        instances += ec2.list(_filter=[('vpc-id', vpc.id)])

    try:
        instance = instances[int(number) - 1]
    except IndexError:
        raise UsageError('There is no instance on number %s' % number)

    return instance


def run_action(number, method, ec2, vpcs):
    instance = get_instance_by_number(number, ec2, vpcs)

    click.echo()
    table_data, i = instance_table_maker(instance)
    table = AsciiTable(table_data)
    click.echo(table.table)
    click.echo()

    instance_id = click.prompt('For safety reason, please enter instance ID: ', type=str)

    if instance.aws.id == instance_id:
        if click.confirm('Do you really want to %s that poor instance?' % method):
            try:
                getattr(instance, method)()
            except exceptions.ClientError:
                if method == 'terminate':
                    if click.confirm(click.style('Termination protection is enabled. \
Do you want to disable it?', fg='red')):
                        instance.disable_api_termination()
                        run_action(number, method, ec2, vpcs)
                else:
                    raise UsageError(str(e))
            except Exception as e:
                raise UsageError(str(e))
    else:
        raise UsageError('Wrong instance ID!')